@@ -125,7 +125,7 @@ class AwsCredentialsProvider(AwsCredentialsProviderBase):
         self._binding = binding
 
     @classmethod
-    def new_default_chain(cls, client_bootstrap):
+    def new_default_chain(cls, client_bootstrap=None):
         """
         Create the default provider chain used by most AWS SDKs.
 
@@ -137,12 +137,16 @@ class AwsCredentialsProvider(AwsCredentialsProviderBase):
         4.  (conditional, on by default) EC2 Instance Metadata
 
         Args:
-            client_bootstrap (ClientBootstrap): Client bootstrap to use when initiating socket connection.
+            client_bootstrap (Optional[ClientBootstrap]): Client bootstrap to use when initiating socket connection.
+                If not set, uses the default static ClientBootstrap instead.
 
         Returns:
             AwsCredentialsProvider:
         """
-        assert isinstance(client_bootstrap, ClientBootstrap)
+        assert isinstance(client_bootstrap, ClientBootstrap) or client_bootstrap is None
+
+        if client_bootstrap is None:
+            client_bootstrap = ClientBootstrap.get_or_create_static_default()
 
         binding = _awscrt.credentials_provider_new_chain_default(client_bootstrap)
         return cls(binding)
@@ -170,7 +174,7 @@ class AwsCredentialsProvider(AwsCredentialsProviderBase):
     @classmethod
     def new_profile(
             cls,
-            client_bootstrap,
+            client_bootstrap=None,
             profile_name=None,
             config_filepath=None,
             credentials_filepath=None):
@@ -179,7 +183,8 @@ class AwsCredentialsProvider(AwsCredentialsProviderBase):
         loaded from the aws credentials file.
 
         Args:
-            client_bootstrap (ClientBootstrap): Client bootstrap to use when initiating socket connection.
+            client_bootstrap (Optional[ClientBootstrap]): Client bootstrap to use when initiating socket connection.
+                If not set, uses the static default ClientBootstrap instead.
 
             profile_name (Optional[str]): Name of profile to use.
                 If not set, uses value from AWS_PROFILE environment variable.
@@ -196,11 +201,14 @@ class AwsCredentialsProvider(AwsCredentialsProviderBase):
         Returns:
             AwsCredentialsProvider:
         """
-        assert isinstance(client_bootstrap, ClientBootstrap)
+        assert isinstance(client_bootstrap, ClientBootstrap) or client_bootstrap is None
         assert isinstance(profile_name, str) or profile_name is None
         assert isinstance(config_filepath, str) or config_filepath is None
         assert isinstance(credentials_filepath, str) or credentials_filepath is None
 
+        if client_bootstrap is None:
+            client_bootstrap = ClientBootstrap.get_or_create_static_default()
+
         binding = _awscrt.credentials_provider_new_profile(
             client_bootstrap, profile_name, config_filepath, credentials_filepath)
         return cls(binding)
@@ -332,7 +340,10 @@ class AwsSigningAlgorithm(IntEnum):
     """AWS signing algorithm enumeration."""
 
     V4 = 0
-    """Use Signature Version 4"""
+    """Signature Version 4"""
+
+    V4_ASYMMETRIC = 1
+    """Signature Version 4 - Asymmetric"""
 
 
 class AwsSignatureType(IntEnum):
@@ -401,9 +412,13 @@ class AwsSigningConfig(NativeResource):
             computed from the signable.
 
         credentials_provider (AwsCredentialsProvider): Credentials provider
-            to fetch signing credentials with.
+            to fetch signing credentials with. If the algorithm is
+            :attr:`AwsSigningAlgorithm.V4_ASYMMETRIC`, ECC-based credentials will be derived from the
+            fetched credentials.
 
-        region (str): The region to sign against.
+        region (str): If the algorithm is :attr:`AwsSigningAlgorithm.V4`, the region to sign against.
+            If the algorithm is :attr:`AwsSigningAlgorithm.V4_ASYMMETRIC`, the value of the
+            "X-amzn-region-set" header (added in signing).
 
         service (str): Name of service to sign a request for.
 
@@ -429,7 +444,7 @@ class AwsSigningConfig(NativeResource):
             encoded). Default is True. All services except S3 use double encoding.
 
         should_normalize_uri_path (bool): Whether the resource paths are
-            normalized when building the canonical request.
+            normalized when building the canonical request. Default is True.
 
         signed_body_value (Optional[str]): If set, this value is used as the
             canonical request's body value. Typically, this is the SHA-256
@@ -550,12 +565,20 @@ class AwsSigningConfig(NativeResource):
 
     @property
     def credentials_provider(self):
-        """AwsCredentialsProvider: Credentials provider to fetch signing credentials with"""
+        """
+        AwsCredentialsProvider: Credentials provider to fetch signing credentials with.
+        If the algorithm is :attr:`AwsSigningAlgorithm.V4_ASYMMETRIC`, ECC-based credentials will be derived
+        from the fetched credentials.
+        """
         return _awscrt.signing_config_get_credentials_provider(self._binding)
 
     @property
     def region(self):
-        """str: The region to sign against"""
+        """
+        str: If signing algorithm is :attr:`AwsSigningAlgorithm.V4`, the region to sign against.
+        If the algorithm is :attr:`AwsSigningAlgorithm.V4_ASYMMETRIC`, the value of the
+        "X-amzn-region-set header" (added in signing).
+        """
         return _awscrt.signing_config_get_region(self._binding)
 
     @property
