@@ -12,6 +12,8 @@
 
 #include <aws/io/logging.h>
 
+#include <Windows.h>
+
 /* The next set of struct definitions are taken directly from the
     windows documentation. We can't include the header files directly
     due to winsock. Also some of the definitions here aren't in the public API
@@ -123,6 +125,10 @@ void aws_overlapped_reset(struct aws_overlapped *overlapped) {
     AWS_ZERO_STRUCT(overlapped->overlapped);
 }
 
+struct _OVERLAPPED *aws_overlapped_to_windows_overlapped(struct aws_overlapped *overlapped) {
+    return (struct _OVERLAPPED *)&overlapped->overlapped;
+}
+
 struct aws_event_loop_vtable s_iocp_vtable = {
     .destroy = s_destroy,
     .run = s_run,
@@ -350,8 +356,10 @@ static int s_run(struct aws_event_loop *event_loop) {
     impl->synced_data.state = EVENT_THREAD_STATE_RUNNING;
 
     AWS_LOGF_INFO(AWS_LS_IO_EVENT_LOOP, "id=%p: Starting event-loop thread.", (void *)event_loop);
+    aws_thread_increment_unjoined_count();
     int err = aws_thread_launch(&impl->thread_created_on, s_event_thread_main, event_loop, &impl->thread_options);
     if (err) {
+        aws_thread_decrement_unjoined_count();
         AWS_LOGF_FATAL(AWS_LS_IO_EVENT_LOOP, "id=%p: thread creation failed.", (void *)event_loop);
         goto clean_up;
     }
@@ -400,6 +408,7 @@ static int s_wait_for_stop_completion(struct aws_event_loop *event_loop) {
 #endif
 
     int err = aws_thread_join(&impl->thread_created_on);
+    aws_thread_decrement_unjoined_count();
     if (err) {
         return AWS_OP_ERR;
     }
