@@ -4,6 +4,7 @@
  */
 
 #include "aws/s3/private/s3_request_messages.h"
+#include "aws/s3/private/s3_checksums.h"
 #include "aws/s3/private/s3_client_impl.h"
 #include "aws/s3/private/s3_meta_request_impl.h"
 #include "aws/s3/private/s3_util.h"
@@ -13,12 +14,15 @@
 #include <aws/common/string.h>
 #include <aws/http/request_response.h>
 #include <aws/io/stream.h>
+#include <aws/io/uri.h>
 #include <aws/s3/s3.h>
 #include <inttypes.h>
 
 const struct aws_byte_cursor g_s3_create_multipart_upload_excluded_headers[] = {
     AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("Content-Length"),
     AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("Content-MD5"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-copy-source"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-copy-source-range"),
 };
 
 const size_t g_s3_create_multipart_upload_excluded_headers_count =
@@ -79,6 +83,8 @@ const struct aws_byte_cursor g_s3_complete_multipart_upload_excluded_headers[] =
     AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-object-lock-mode"),
     AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-object-lock-retain-until-date"),
     AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-object-lock-legal-hold"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-copy-source"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-copy-source-range"),
 };
 
 const size_t g_s3_complete_multipart_upload_excluded_headers_count =
@@ -111,6 +117,8 @@ const struct aws_byte_cursor g_s3_abort_multipart_upload_excluded_headers[] = {
     AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-object-lock-mode"),
     AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-object-lock-retain-until-date"),
     AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-object-lock-legal-hold"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-copy-source"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-copy-source-range"),
 };
 
 const size_t g_s3_abort_multipart_upload_excluded_headers_count =
@@ -131,7 +139,7 @@ struct aws_http_message *aws_s3_ranged_get_object_message_new(
     AWS_PRECONDITION(allocator);
     AWS_PRECONDITION(base_message);
 
-    struct aws_http_message *message = aws_s3_message_util_copy_http_message(allocator, base_message, NULL, 0);
+    struct aws_http_message *message = aws_s3_message_util_copy_http_message_no_body(allocator, base_message, NULL, 0);
 
     if (message == NULL) {
         return NULL;
@@ -156,11 +164,12 @@ error_clean_up:
 /* Creates a create-multipart-upload request from a given put objet request. */
 struct aws_http_message *aws_s3_create_multipart_upload_message_new(
     struct aws_allocator *allocator,
-    struct aws_http_message *base_message) {
+    struct aws_http_message *base_message,
+    enum aws_s3_checksum_algorithm algorithm) {
     AWS_PRECONDITION(allocator);
 
     /* For multipart upload, sse related headers should only be shown in create-multipart request */
-    struct aws_http_message *message = aws_s3_message_util_copy_http_message(
+    struct aws_http_message *message = aws_s3_message_util_copy_http_message_no_body(
         allocator,
         base_message,
         g_s3_create_multipart_upload_excluded_headers,
@@ -185,6 +194,14 @@ struct aws_http_message *aws_s3_create_multipart_upload_message_new(
             goto error_clean_up;
         }
     }
+    if (algorithm) {
+        if (aws_http_headers_set(
+                headers,
+                g_create_mpu_checksum_header_name,
+                *aws_get_create_mpu_header_name_from_algorithm(algorithm))) {
+            goto error_clean_up;
+        }
+    }
 
     aws_http_message_set_request_method(message, g_post_method);
     aws_http_message_set_body_stream(message, NULL);
@@ -209,12 +226,14 @@ struct aws_http_message *aws_s3_upload_part_message_new(
     struct aws_byte_buf *buffer,
     uint32_t part_number,
     const struct aws_string *upload_id,
-    bool should_compute_content_md5) {
+    bool should_compute_content_md5,
+    const enum aws_s3_checksum_algorithm checksum_algorithm,
+    struct aws_byte_buf *encoded_checksum_output) {
     AWS_PRECONDITION(allocator);
     AWS_PRECONDITION(base_message);
     AWS_PRECONDITION(part_number > 0);
 
-    struct aws_http_message *message = aws_s3_message_util_copy_http_message(
+    struct aws_http_message *message = aws_s3_message_util_copy_http_message_no_body(
         allocator, base_message, g_s3_upload_part_excluded_headers, AWS_ARRAY_SIZE(g_s3_upload_part_excluded_headers));
 
     if (message == NULL) {
@@ -226,11 +245,12 @@ struct aws_http_message *aws_s3_upload_part_message_new(
     }
 
     if (buffer != NULL) {
-        if (aws_s3_message_util_assign_body(allocator, buffer, message) == NULL) {
+
+        if (aws_s3_message_util_assign_body(allocator, buffer, message, checksum_algorithm, encoded_checksum_output) ==
+            NULL) {
             goto error_clean_up;
         }
-
-        if (should_compute_content_md5) {
+        if (checksum_algorithm == AWS_SCA_NONE && should_compute_content_md5) {
             if (aws_s3_message_util_add_content_md5_header(allocator, buffer, message)) {
                 goto error_clean_up;
             }
@@ -251,6 +271,189 @@ error_clean_up:
     return NULL;
 }
 
+struct aws_http_message *aws_s3_upload_part_copy_message_new(
+    struct aws_allocator *allocator,
+    struct aws_http_message *base_message,
+    struct aws_byte_buf *buffer,
+    uint32_t part_number,
+    uint64_t range_start,
+    uint64_t range_end,
+    const struct aws_string *upload_id,
+    bool should_compute_content_md5) {
+    AWS_PRECONDITION(allocator);
+    AWS_PRECONDITION(base_message);
+    AWS_PRECONDITION(part_number > 0);
+
+    struct aws_http_message *message = aws_s3_message_util_copy_http_message_no_body(
+        allocator, base_message, g_s3_upload_part_excluded_headers, AWS_ARRAY_SIZE(g_s3_upload_part_excluded_headers));
+
+    if (message == NULL) {
+        goto error_clean_up;
+    }
+
+    if (aws_s3_message_util_set_multipart_request_path(allocator, upload_id, part_number, false, message)) {
+        goto error_clean_up;
+    }
+
+    if (buffer != NULL) {
+        /* part copy does not have a ChecksumAlgorithm member, it will use the same algorithm as the create
+         * multipart upload request specifies */
+        if (aws_s3_message_util_assign_body(allocator, buffer, message, AWS_SCA_NONE, NULL /* out_checksum */) ==
+            NULL) {
+            goto error_clean_up;
+        }
+
+        if (should_compute_content_md5) {
+            if (aws_s3_message_util_add_content_md5_header(allocator, buffer, message)) {
+                goto error_clean_up;
+            }
+        }
+    }
+
+    char source_range[1024];
+    snprintf(source_range, sizeof(source_range), "bytes=%" PRIu64 "-%" PRIu64, range_start, range_end);
+
+    struct aws_http_header source_range_header = {
+        .name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-copy-source-range"),
+        .value = aws_byte_cursor_from_c_str(source_range),
+    };
+
+    struct aws_http_headers *headers = aws_http_message_get_headers(message);
+    aws_http_headers_add_header(headers, &source_range_header);
+
+    return message;
+
+error_clean_up:
+
+    if (message != NULL) {
+        aws_http_message_release(message);
+        message = NULL;
+    }
+
+    return NULL;
+}
+
+/* Creates a HEAD GetObject request to get the size of the specified object. */
+struct aws_http_message *aws_s3_get_object_size_message_new(
+    struct aws_allocator *allocator,
+    struct aws_http_message *base_message,
+    struct aws_byte_cursor source_bucket,
+    struct aws_byte_cursor source_key) {
+
+    (void)base_message;
+
+    AWS_PRECONDITION(allocator);
+
+    struct aws_http_message *message = aws_http_message_new_request(allocator);
+
+    if (message == NULL) {
+        return NULL;
+    }
+
+    const struct aws_byte_cursor head_operation = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("HEAD");
+    if (aws_http_message_set_request_method(message, head_operation)) {
+        goto error_clean_up;
+    }
+
+    char destination_path[1024];
+    snprintf(destination_path, sizeof(destination_path), "/%.*s", (int)source_key.len, source_key.ptr);
+    /* TODO: url encode */
+
+    if (aws_http_message_set_request_path(message, aws_byte_cursor_from_c_str(destination_path))) {
+        goto error_clean_up;
+    }
+
+    char host_header_value[1024];
+    snprintf(
+        host_header_value,
+        sizeof(host_header_value),
+        "%.*s.s3.us-west-2.amazonaws.com",
+        (int)source_bucket.len,
+        source_bucket.ptr);
+    struct aws_http_header host_header = {
+        .name = g_host_header_name,
+        .value = aws_byte_cursor_from_c_str(host_header_value),
+    };
+    aws_http_message_add_header(message, host_header);
+
+    aws_http_message_set_body_stream(message, NULL);
+
+    return message;
+
+error_clean_up:
+
+    if (message != NULL) {
+        aws_http_message_release(message);
+        message = NULL;
+    }
+
+    return NULL;
+}
+
+/* Creates a HEAD GetObject sub-request to get the size of the source object of a Copy meta request. */
+struct aws_http_message *aws_s3_get_source_object_size_message_new(
+    struct aws_allocator *allocator,
+    struct aws_http_message *base_message) {
+    AWS_PRECONDITION(allocator);
+
+    struct aws_http_message *message = NULL;
+
+    /* find the x-amz-copy-source header */
+    struct aws_http_headers *headers = aws_http_message_get_headers(base_message);
+
+    struct aws_byte_cursor source_header_value;
+    AWS_ZERO_STRUCT(source_header_value);
+
+    const struct aws_byte_cursor copy_source_header = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-copy-source");
+    if (aws_http_headers_get(headers, copy_source_header, &source_header_value) != AWS_OP_SUCCESS) {
+        AWS_LOGF_ERROR(AWS_LS_S3_GENERAL, "CopyRequest is missing the x-amz-copy-source header");
+        return NULL;
+    }
+
+    /* url decode */
+    struct aws_byte_buf decode_buffer;
+    AWS_ZERO_STRUCT(decode_buffer);
+    aws_byte_buf_init(&decode_buffer, allocator, 0);
+    if (aws_byte_buf_append_decoding_uri(&decode_buffer, &source_header_value) != AWS_OP_SUCCESS) {
+        goto error_cleanup;
+    }
+
+    /* extracts source bucket and key */
+    struct aws_byte_cursor source_bucket = aws_byte_cursor_from_buf(&decode_buffer);
+
+    /* x-amz-copy-source might have an optional leading slash. if so let's skip it */
+    if (decode_buffer.len > 1 && decode_buffer.buffer[0] == '/') {
+        /* skip the leading slash */
+        aws_byte_cursor_advance(&source_bucket, 1);
+    }
+
+    /* as we skipped the optional leading slash, from this point source format is always {bucket}/{key}. split them.
+     */
+    struct aws_byte_cursor source_key = source_bucket;
+
+    while (source_key.len > 0) {
+        if (*source_key.ptr == '/') {
+            source_bucket.len = source_key.ptr - source_bucket.ptr;
+            aws_byte_cursor_advance(&source_key, 1); /* skip the / between bucket and key */
+            break;
+        }
+        aws_byte_cursor_advance(&source_key, 1);
+    }
+
+    if (source_bucket.len == 0 || source_key.len == 0) {
+        AWS_LOGF_ERROR(
+            AWS_LS_S3_GENERAL,
+            "The CopyRequest x-amz-copy-source header must contain the bucket and object key separated by a slash");
+        goto error_cleanup;
+    }
+
+    message = aws_s3_get_object_size_message_new(allocator, base_message, source_bucket, source_key);
+
+error_cleanup:
+    aws_byte_buf_clean_up(&decode_buffer);
+    return message;
+}
+
 static const struct aws_byte_cursor s_complete_payload_begin = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL(
     "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
     "<CompleteMultipartUpload xmlns=\"http://s3.amazonaws.com/doc/2006-03-01/\">\n");
@@ -265,23 +468,30 @@ static const struct aws_byte_cursor s_part_section_string_1 =
     AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("</ETag>\n"
                                           "         <PartNumber>");
 
-static const struct aws_byte_cursor s_part_section_string_2 = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("</PartNumber>\n"
-                                                                                                    "    </Part>\n");
-
+static const struct aws_byte_cursor s_close_part_number_tag = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("</PartNumber>\n");
+static const struct aws_byte_cursor s_close_part_tag = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("    </Part>\n");
+static const struct aws_byte_cursor s_open_start_bracket = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("         <");
+static const struct aws_byte_cursor s_open_end_bracket = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("</");
+static const struct aws_byte_cursor s_close_bracket = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL(">");
+static const struct aws_byte_cursor s_close_bracket_new_line = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL(">\n");
 /* Create a complete-multipart message, which includes an XML payload of all completed parts. */
 struct aws_http_message *aws_s3_complete_multipart_message_new(
     struct aws_allocator *allocator,
     struct aws_http_message *base_message,
     struct aws_byte_buf *body_buffer,
     const struct aws_string *upload_id,
-    const struct aws_array_list *etags) {
+    const struct aws_array_list *etags,
+    struct aws_byte_buf *checksums,
+    enum aws_s3_checksum_algorithm algorithm) {
     AWS_PRECONDITION(allocator);
     AWS_PRECONDITION(base_message);
     AWS_PRECONDITION(body_buffer);
     AWS_PRECONDITION(upload_id);
     AWS_PRECONDITION(etags);
 
-    struct aws_http_message *message = aws_s3_message_util_copy_http_message(
+    const struct aws_byte_cursor *mpu_algorithm_checksum_name = aws_get_complete_mpu_name_from_algorithm(algorithm);
+
+    struct aws_http_message *message = aws_s3_message_util_copy_http_message_no_body(
         allocator,
         base_message,
         g_s3_complete_multipart_upload_excluded_headers,
@@ -344,7 +554,35 @@ struct aws_http_message *aws_s3_complete_multipart_message_new(
                 goto error_clean_up;
             }
 
-            if (aws_byte_buf_append_dynamic(body_buffer, &s_part_section_string_2)) {
+            if (aws_byte_buf_append_dynamic(body_buffer, &s_close_part_number_tag)) {
+                goto error_clean_up;
+            }
+            if (mpu_algorithm_checksum_name) {
+                struct aws_byte_cursor checksum = aws_byte_cursor_from_buf(&checksums[etag_index]);
+
+                if (aws_byte_buf_append_dynamic(body_buffer, &s_open_start_bracket)) {
+                    goto error_clean_up;
+                }
+                if (aws_byte_buf_append_dynamic(body_buffer, mpu_algorithm_checksum_name)) {
+                    goto error_clean_up;
+                }
+                if (aws_byte_buf_append_dynamic(body_buffer, &s_close_bracket)) {
+                    goto error_clean_up;
+                }
+                if (aws_byte_buf_append_dynamic(body_buffer, &checksum)) {
+                    goto error_clean_up;
+                }
+                if (aws_byte_buf_append_dynamic(body_buffer, &s_open_end_bracket)) {
+                    goto error_clean_up;
+                }
+                if (aws_byte_buf_append_dynamic(body_buffer, mpu_algorithm_checksum_name)) {
+                    goto error_clean_up;
+                }
+                if (aws_byte_buf_append_dynamic(body_buffer, &s_close_bracket_new_line)) {
+                    goto error_clean_up;
+                }
+            }
+            if (aws_byte_buf_append_dynamic(body_buffer, &s_close_part_tag)) {
                 goto error_clean_up;
             }
         }
@@ -353,7 +591,7 @@ struct aws_http_message *aws_s3_complete_multipart_message_new(
             goto error_clean_up;
         }
 
-        aws_s3_message_util_assign_body(allocator, body_buffer, message);
+        aws_s3_message_util_assign_body(allocator, body_buffer, message, AWS_SCA_NONE, NULL /* out_checksum */);
     }
 
     return message;
@@ -375,7 +613,7 @@ struct aws_http_message *aws_s3_abort_multipart_upload_message_new(
     struct aws_http_message *base_message,
     const struct aws_string *upload_id) {
 
-    struct aws_http_message *message = aws_s3_message_util_copy_http_message(
+    struct aws_http_message *message = aws_s3_message_util_copy_http_message_no_body(
         allocator,
         base_message,
         g_s3_abort_multipart_upload_excluded_headers,
@@ -404,7 +642,9 @@ error_clean_up:
 struct aws_input_stream *aws_s3_message_util_assign_body(
     struct aws_allocator *allocator,
     struct aws_byte_buf *byte_buf,
-    struct aws_http_message *out_message) {
+    struct aws_http_message *out_message,
+    enum aws_s3_checksum_algorithm algorithm,
+    struct aws_byte_buf *out_checksum) {
     AWS_PRECONDITION(allocator);
     AWS_PRECONDITION(out_message);
     AWS_PRECONDITION(byte_buf);
@@ -422,13 +662,42 @@ struct aws_input_stream *aws_s3_message_util_assign_body(
         goto error_clean_up;
     }
 
+    if (algorithm) {
+        /* set Content-Encoding header */
+        if (aws_http_headers_set(headers, g_content_encoding_header_name, g_content_encoding_header_aws_chunked)) {
+            goto error_clean_up;
+        }
+        /* set x-amz-trailer header */
+        if (aws_http_headers_set(headers, g_trailer_header_name, *aws_get_http_header_name_from_algorithm(algorithm))) {
+            goto error_clean_up;
+        }
+        /* set x-amz-decoded-content-length header */
+        char decoded_content_length_buffer[64] = "";
+        snprintf(
+            decoded_content_length_buffer,
+            sizeof(decoded_content_length_buffer),
+            "%" PRIu64,
+            (uint64_t)buffer_byte_cursor.len);
+        struct aws_byte_cursor decode_content_length_cursor =
+            aws_byte_cursor_from_array(decoded_content_length_buffer, strlen(decoded_content_length_buffer));
+        if (aws_http_headers_set(headers, g_decoded_content_length_header_name, decode_content_length_cursor)) {
+            goto error_clean_up;
+        }
+        /* set input stream to chunk stream */
+        input_stream = aws_chunk_stream_new(allocator, input_stream, algorithm, out_checksum);
+        if (!input_stream) {
+            goto error_clean_up;
+        }
+    }
+    int64_t stream_length = 0;
+    if (aws_input_stream_get_length(input_stream, &stream_length)) {
+        goto error_clean_up;
+    }
     char content_length_buffer[64] = "";
-    snprintf(content_length_buffer, sizeof(content_length_buffer), "%" PRIu64, (uint64_t)buffer_byte_cursor.len);
+    snprintf(content_length_buffer, sizeof(content_length_buffer), "%" PRIu64, (uint64_t)stream_length);
     struct aws_byte_cursor content_length_cursor =
         aws_byte_cursor_from_array(content_length_buffer, strlen(content_length_buffer));
-
     if (aws_http_headers_set(headers, g_content_length_header_name, content_length_cursor)) {
-        aws_input_stream_destroy(input_stream);
         goto error_clean_up;
     }
 
@@ -437,7 +706,7 @@ struct aws_input_stream *aws_s3_message_util_assign_body(
     return input_stream;
 
 error_clean_up:
-
+    AWS_LOGF_ERROR(AWS_LS_S3_CLIENT, "Failed to assign body for s3 request http message, from body buffer .");
     aws_input_stream_destroy(input_stream);
     return NULL;
 }
@@ -486,8 +755,8 @@ error_clean_up:
     return AWS_OP_ERR;
 }
 
-/* Copy an existing HTTP message's headers and body. */
-struct aws_http_message *aws_s3_message_util_copy_http_message(
+/* Copy an existing HTTP message's headers, method, and path. */
+struct aws_http_message *aws_s3_message_util_copy_http_message_no_body(
     struct aws_allocator *allocator,
     struct aws_http_message *base_message,
     const struct aws_byte_cursor *excluded_header_array,
