@@ -36,6 +36,13 @@ impl Connection {
     pub fn new(mode: s2n_mode::Type) -> Self {
         crate::raw::init::init();
         let connection = unsafe { s2n_connection_new(mode).into_result() }.unwrap();
+
+        unsafe {
+            let mut config = core::ptr::null_mut();
+            debug_assert! {
+                s2n_connection_get_config(connection.as_ptr(), &mut config).into_result().is_err()
+            };
+        }
         Self {
             connection,
             config: None,
@@ -78,6 +85,12 @@ impl Connection {
         unsafe {
             s2n_connection_set_config(self.connection.as_ptr(), config.as_mut_ptr()).into_result()
         }?;
+        unsafe {
+            let mut config = core::ptr::null_mut();
+            debug_assert! {
+                s2n_connection_get_config(self.connection.as_ptr(), &mut config).into_result().is_ok()
+            };
+        }
         self.config = Some(config);
         Ok(self)
     }
@@ -121,7 +134,7 @@ impl Connection {
     /// Client Hello message as the ALPN extension. As an S2N_SERVER, the list is used to negotiate
     /// a mutual application protocol with the client. After the negotiation for the connection has
     /// completed, the agreed upon protocol can be retrieved with s2n_get_application_protocol
-    pub fn set_alpn_preference<P: IntoIterator<Item = I>, I: AsRef<[u8]>>(
+    pub fn set_application_protocol_preference<P: IntoIterator<Item = I>, I: AsRef<[u8]>>(
         &mut self,
         protocols: P,
     ) -> Result<&mut Self, Error> {
@@ -132,13 +145,16 @@ impl Connection {
         }?;
 
         for protocol in protocols {
-            self.append_alpn_preference(protocol.as_ref())?;
+            self.append_application_protocol_preference(protocol.as_ref())?;
         }
 
         Ok(self)
     }
 
-    pub fn append_alpn_preference(&mut self, protocol: &[u8]) -> Result<&mut Self, Error> {
+    pub fn append_application_protocol_preference(
+        &mut self,
+        protocol: &[u8],
+    ) -> Result<&mut Self, Error> {
         unsafe {
             s2n_connection_append_protocol_preference(
                 self.connection.as_ptr(),
@@ -234,8 +250,8 @@ impl Connection {
         Some(alert as u8)
     }
 
-    /// Sets the SNI value for the connection
-    pub fn set_sni(&mut self, sni: &[u8]) -> Result<&mut Self, Error> {
+    /// Sets the server name value for the connection
+    pub fn set_server_name(&mut self, sni: &[u8]) -> Result<&mut Self, Error> {
         let sni = std::ffi::CString::new(sni).map_err(|_| Error::InvalidInput)?;
         unsafe { s2n_set_server_name(self.connection.as_ptr(), sni.as_ptr()).into_result() }?;
         Ok(self)
@@ -244,6 +260,11 @@ impl Connection {
 
 #[cfg(feature = "quic")]
 impl Connection {
+    pub fn enable_quic(&mut self) -> Result<&mut Self, Error> {
+        unsafe { s2n_connection_enable_quic(self.connection.as_ptr()).into_result() }?;
+        Ok(self)
+    }
+
     pub fn set_quic_transport_parameters(&mut self, buffer: &[u8]) -> Result<&mut Self, Error> {
         unsafe {
             s2n_connection_set_quic_transport_parameters(
