@@ -230,13 +230,15 @@ int s2n_config_set_unsafe_for_testing(struct s2n_config *config)
 
 int s2n_config_defaults_init(void)
 {
-    /* Set up default */
-    POSIX_GUARD(s2n_config_init(&s2n_default_config));
-    POSIX_GUARD(s2n_config_setup_default(&s2n_default_config));
-
     /* Set up fips defaults */
-    POSIX_GUARD(s2n_config_init(&s2n_default_fips_config));
-    POSIX_GUARD(s2n_config_setup_fips(&s2n_default_fips_config));
+    if (s2n_is_in_fips_mode()) {
+        POSIX_GUARD(s2n_config_init(&s2n_default_fips_config));
+        POSIX_GUARD(s2n_config_setup_fips(&s2n_default_fips_config));
+    } else {
+        /* Set up default */
+        POSIX_GUARD(s2n_config_init(&s2n_default_config));
+        POSIX_GUARD(s2n_config_setup_default(&s2n_default_config));
+    }
 
     /* Set up TLS 1.3 defaults */
     POSIX_GUARD(s2n_config_init(&s2n_default_tls13_config));
@@ -247,8 +249,8 @@ int s2n_config_defaults_init(void)
 
 void s2n_wipe_static_configs(void)
 {
-    s2n_config_cleanup(&s2n_default_config);
     s2n_config_cleanup(&s2n_default_fips_config);
+    s2n_config_cleanup(&s2n_default_config);
     s2n_config_cleanup(&s2n_default_tls13_config);
 }
 
@@ -332,6 +334,14 @@ int s2n_config_free_dhparams(struct s2n_config *config)
     return 0;
 }
 
+S2N_CLEANUP_RESULT s2n_config_ptr_free(struct s2n_config **config)
+{
+    RESULT_ENSURE_REF(config);
+    RESULT_GUARD_POSIX(s2n_config_free(*config));
+    *config = NULL;
+    return S2N_RESULT_OK;
+}
+
 int s2n_config_free(struct s2n_config *config)
 {
     s2n_config_cleanup(config);
@@ -897,3 +907,20 @@ int s2n_config_set_async_pkey_validation_mode(struct s2n_config *config, s2n_asy
 
     POSIX_BAIL(S2N_ERR_INVALID_ARGUMENT);
 }
+
+int s2n_config_set_ctx(struct s2n_config *config, void *ctx) {
+    POSIX_ENSURE_REF(config);
+
+    config->context = ctx;
+
+    return S2N_SUCCESS;
+}
+
+int s2n_config_get_ctx(struct s2n_config *config, void **ctx) {
+    POSIX_ENSURE_REF(config);
+    POSIX_ENSURE_REF(ctx);
+
+    *ctx = config->context;
+
+    return S2N_SUCCESS;
+}
