@@ -30,20 +30,22 @@
 struct s2n_cipher_preferences;
 
 struct s2n_config {
+    unsigned use_tickets:1;
+
+    /* Whether a connection can be used by a QUIC implementation.
+     * See s2n_quic_support.h */
+    unsigned quic_enabled:1;
+
     unsigned cert_allocated:1;
     unsigned default_certs_are_explicit:1;
-    unsigned use_tickets:1;
     unsigned use_session_cache:1;
     /* if this is FALSE, server will ignore client's Maximum Fragment Length request */
     unsigned accept_mfl:1;
     unsigned check_ocsp:1;
     unsigned disable_x509_validation:1;
     unsigned max_verify_cert_chain_depth_set:1;
-    /* Whether a connection can be used by a QUIC implementation.
-     * See s2n_quic_support.h */
-    unsigned quic_enabled:1;
     /* Whether to add dss cert type during a server certificate request.
-     * See https://github.com/awslabs/s2n/blob/main/docs/USAGE-GUIDE.md */
+     * See https://github.com/aws/s2n-tls/blob/main/docs/USAGE-GUIDE.md */
     unsigned cert_req_dss_legacy_compat_enabled:1;
     /* Whether any RSA certificates have been configured server-side to send to clients. This is needed so that the
      * server knows whether or not to self-downgrade to TLS 1.2 if the server is compiled with Openssl 1.0.2 and does
@@ -131,8 +133,13 @@ struct s2n_config {
     s2n_psk_mode psk_mode;
 
     s2n_async_pkey_validation_mode async_pkey_validation_mode;
+
+    /* The user defined context associated with config */
+    void *context;
 };
 
+S2N_CLEANUP_RESULT s2n_config_ptr_free(struct s2n_config **config);
+
 int s2n_config_defaults_init(void);
 extern struct s2n_config *s2n_fetch_default_config(void);
 int s2n_config_set_unsafe_for_testing(struct s2n_config *config);
