@@ -27,12 +27,13 @@
 #include "utils/s2n_map.h"
 #include "utils/s2n_map_internal.h"
 
-#include <s2n.h>
+#include "api/s2n.h"
 
 #define S2N_INITIAL_TABLE_SIZE 1024
 
 static S2N_RESULT s2n_map_slot(const struct s2n_map *map, struct s2n_blob *key, uint32_t *slot)
 {
+    RESULT_ENSURE_REF(map);
     union {
         uint8_t u8[32];
         uint32_t u32[8];
@@ -50,6 +51,7 @@ static S2N_RESULT s2n_map_slot(const struct s2n_map *map, struct s2n_blob *key,
 
 static S2N_RESULT s2n_map_embiggen(struct s2n_map *map, uint32_t capacity)
 {
+    RESULT_ENSURE_REF(map);
     struct s2n_blob mem = {0};
     struct s2n_map tmp = {0};
 
@@ -107,6 +109,7 @@ struct s2n_map *s2n_map_new_with_initial_capacity(uint32_t capacity)
 
 S2N_RESULT s2n_map_add(struct s2n_map *map, struct s2n_blob *key, struct s2n_blob *value)
 {
+    RESULT_ENSURE_REF(map);
     RESULT_ENSURE(!map->immutable, S2N_ERR_MAP_IMMUTABLE);
 
     if (map->capacity < (map->size * 2)) {
@@ -139,6 +142,7 @@ S2N_RESULT s2n_map_add(struct s2n_map *map, struct s2n_blob *key, struct s2n_blo
 
 S2N_RESULT s2n_map_put(struct s2n_map *map, struct s2n_blob *key, struct s2n_blob *value)
 {
+    RESULT_ENSURE_REF(map);
     RESULT_ENSURE(!map->immutable, S2N_ERR_MAP_IMMUTABLE);
 
     if (map->capacity < (map->size * 2)) {
@@ -174,6 +178,7 @@ S2N_RESULT s2n_map_put(struct s2n_map *map, struct s2n_blob *key, struct s2n_blo
 
 S2N_RESULT s2n_map_complete(struct s2n_map *map)
 {
+    RESULT_ENSURE_REF(map);
     map->immutable = 1;
 
     return S2N_RESULT_OK;
@@ -181,6 +186,7 @@ S2N_RESULT s2n_map_complete(struct s2n_map *map)
 
 S2N_RESULT s2n_map_unlock(struct s2n_map *map)
 {
+    RESULT_ENSURE_REF(map);
     map->immutable = 0;
 
     return S2N_RESULT_OK;
@@ -188,6 +194,7 @@ S2N_RESULT s2n_map_unlock(struct s2n_map *map)
 
 S2N_RESULT s2n_map_lookup(const struct s2n_map *map, struct s2n_blob *key, struct s2n_blob *value, bool *key_found)
 {
+    RESULT_ENSURE_REF(map);
     RESULT_ENSURE(map->immutable, S2N_ERR_MAP_MUTABLE);
 
     uint32_t slot = 0;
@@ -222,7 +229,13 @@ S2N_RESULT s2n_map_lookup(const struct s2n_map *map, struct s2n_blob *key, struc
 
 S2N_RESULT s2n_map_free(struct s2n_map *map)
 {
+    if (map == NULL) {
+        return S2N_RESULT_OK;
+    }
+
     /* Free the keys and values */
+    /* cppcheck has a false positive warning for checking the pointer here */
+    /* cppcheck-suppress nullPointerRedundantCheck */
     for (uint32_t i = 0; i < map->capacity; i++) {
         if (map->table[i].key.size) {
             RESULT_GUARD_POSIX(s2n_free(&map->table[i].key));
