@@ -14,7 +14,7 @@
  */
 #pragma once
 
-#include <s2n.h>
+#include "api/s2n.h"
 #include "utils/s2n_result.h"
 #include "utils/s2n_array.h"
 
